function subset = sustainedUnsustainedSubsets(odeResults)
% 
% Split the result of a predator-prey ODE model result into the following
% subsets.
% sustained: The prey count at the end of the simulation is >= half the
%            initial prey count.
%
% unsustained: The prey count at the end of the simulation is < half the
%              initial prey count.
%
% In comments below:
%
% R: The number of runs.
%
% P: The number of varied parameters and varied initial conditions.
%unSustainedRuns
% O: The number of model outputs.
%
% TP: The number of timepoints.
%
% Inputs:
%     odeResults: A structure containing the result of running the
%                 predator-prey model.
%                 odeResults.modelOutput is an O x R x TP matrix.
%                 odeResults.lhsMatrix is an R x P matrix.

%
% Outputs:
%     subset: A cell array with one element for each subset created.
%             Each cell array element has a copy of odeResults.modelOutput and
%             of odeResults.lhsMatrix, but with information for only those
%             runs in the subset. Ex. subset{1}.modelOutput will be an OxS1xTP
%             matrix where S1 is the number of runs in that subset and
%             subset{1}.lhsMatrix will be an S1xP.


% Get the count of prey for each run at the end of the run.
startPreyCount = squeeze(odeResults.modelOutput(1,:,1));
endPreyCount = squeeze(odeResults.modelOutput(1,:,end));

% The indices for runs where the ending prey count is >= half the ending
% prey count.
sustainedRuns = endPreyCount >= 0.5 .* startPreyCount;

% The indices for runs where the ending prey count is >= half the ending
% prey count.
unSustainedRuns = endPreyCount < 0.5 .* startPreyCount;

subset{1,1}.name = 'sustained';
subset{2,1}.name = 'unsustained';

subset{1,1}.lhsMatrix = odeResults.lhsMatrix(sustainedRuns, :);
subset{2,1}.lhsMatrix = odeResults.lhsMatrix(unSustainedRuns, :);

subset{1,1}.modelOutput = odeResults.modelOutput(:, sustainedRuns, :);
subset{2,1}.modelOutput = odeResults.modelOutput(:, unSustainedRuns, :);

end % function controlledUncontrolledSubsets
