function lhs_ode_run_new(lhsODESettingsFileName)

% -------------------------------------------------------------------------
%
% Setup and run of an LHS analysis of an ODE model - main routine
%
% In comments below:
%
% TP is the number of timepoints.
%
% R is number of runs, same as NR in the settings file.
%
% P is the number of varied parameters and varied initial conditions.
%
% O is the number of of model outputs, the same as the number of model
%   equations.
%
% Input:
%
% lhsOdeSettingsFileName: a file that contains the definitions of the LHS
%                         settings needed by this script. If not specified
%                         as a function argument then retrieved using a
%                         file browser dialog.
%
% See the header comments in a settings file for a description of the members
% of the structure named "odeSettings" returned after processing a settings
% file.
%
% Output
%
%     Structure result in the Matlab workspace, which can be optionally
%     saved in a .mat file via a file save dialog,
%
%     result has the following fields:
%
%     error_runs: An N x 3 cell array with one row for each time a model run
%                 failed to solve. Each row has 3 items - the run number of a
%                 run that failed, an error message and an error number.
%
%     vcInfo: A structure with version control information for the version
%             control project this script is part of.
%
%     odeSettings: A structure with fields defined from processing the
%                  settings file.
%
%     paramMatrix: An R x PA matrix of all the parameter values for the model,
%                  whether varied or not. Row i is the parameters used for run
%                  i.
%
%     icMatrix: An R x O matrix of the initial conditions. Row i is the
%               initial conditions for run i.
%
%     pulseMatrix: If pulsing is performed, an R x O matrix of the pulse
%                  values. Row i is the pulse values for run i.
%
%     lhsMatrix: An R x P matrix of varied parameter and varied initial
%                condition values.
%
%     paramNames: A 1 X P matrix of parameter names.
%
%     modelOutput: A O x R x TP matrix of model output values.
%
%     The following fields are only defined if PRCC analysis is performed
%     (if alpha is defined in the settings file, and therefore defined in
%     odeSettings).
%
%     analysisTimePoints: A TP x 1 vector of timestep values.
%                         A copy of the same field in odeSettings.
%
%     alpha: If specified, the alpha value to use for determining significance
%            in PRCC analysis. A copy of the same field in odeSettings.
%
%     modelOutputNames: An O x 1 vector of names for the model output.
%                       a copy of odeSettings.icNames.
%
%     uncorrectedPrcc: A TP x O x P matrix of the PRCC values.
%
%     uncorrectedSignificance: A TP x O x P matrix of the significance values
%                              for the PRCC values in uncorrectedPrcc.
%
%     uncorrectedSignificantPrcc: A TP x O x P matrix of the PRCC values.
%                                 The same as uncorrectedPrcc, except entries
%                                 for PRCC values that are not significant have
%                                 been set to 1.
%
%     uncorrectedSignificantSignifcance: A TP x O x P matrix. The same as
%                                        uncorrectedSignificance, except that
%                                        significance values for PRCC values
%                                        that are not significant have been
%                                        set to 1.
%
%     bonferroniSignificance: A TP x O x P matrix of signficance values
%                             corrected using the Bonferroni correction.
%
%     bonferroniSignificantPrcc: A TP x O x P matrix of PRCC values.
%                                The same as uncorrectedPrcc, except entries
%                                for PRCC values that are not significant 
%                                according to alpha and bonferroniSignificance
%                                have been set to 1.
%
%     bonferroniSignificantSignifcance: A TP x O x P matrix. The same as
%                                       uncorrectedSignificance, except that
%                                       bonferroniSignificance values that
%                                       are not significant have been set to 1.
%
%     bonferroniSignificance: A TP x O x P matrix of signficance values
%                             corrected using the Bonferroni correction.
%
%     bonferroniSignificantPrcc: A TP x O x P matrix of PRCC values.
%                                The same as uncorrectedPrcc, except entries
%                                for PRCC values that are not significant 
%                                according to alpha and bonferroniSignificance
%                                have been set to 1.
%
%     bonferroniSignificantSignifcance: A TP x O x P matrix. The same as
%                                       uncorrectedSignificance, except that
%                                       bonferroniSignificance values that
%
%--------------------------------------------------------------------------

% Declare variables that can be saved to a .mat file. They must be also
% assigned to the base workspace, otherwise the cleanup function that does
% the saving won't be able to access them. Each variable to be saved must be
% declared and assigned here, since not all will be defined later,
% depending on program options (ex. a PRCC might not be performed). Also
% each variable must be assigned again if it is defined later, so the most
% current value gets saved - assigin essentially copies the current value
% to the base workspace, it doesn't cause any future values to be available
% in the base workspace.
                                
% error_runs is for handling the case where the Matlab ode solver fails for a
% set of parameters and initial conditions. A solver can fail if it can't
% find a sufficiently small time step, if it encounters a singular matrix
% or if it takes longer than our defined solver time limit.
result.error_runs={};

% Get the scripts directory version control info, so we can know which version
% of the scripts directory was used for this run, to aid reproducibility.  Use
% this script's name as the output prefix, to distinuish the version control
% information for a run of this script with version control information from a
% run of another script.
callerFullPath = mfilename('fullpath');
result.vcInfo = saveVersionControlInfo(callerFullPath, '', 'lhs_ode_run_new.m');

assignin('base', 'result', result);


cleanupObj = onCleanup(@cleanMeUp);

fprintf('\n=====================================\n');
fprintf('   ODE LHS Run');
fprintf('\n=====================================\n');


% Load the settings file - everything returned in the odeSettings structure.
result.odeSettings = lhs_ode_get_run_settings_new(lhsODESettingsFileName);
assignin('base', 'result', result);

% Build the parameter matrix and initial condition matrix.
%
% The ith row of the paramter matrix are the parameters to use for run i.
%
% The ith row of the initial condition matrix are the initial conditions to use
% for run i.

[result.paramMatrix, result.icMatrix, result.pulseMatrix, result.lhsMatrix, ...
 result.paramNames] = lhs_ode_define_run_matrices_new(result.odeSettings);
assignin('base', 'result', result);

% The function to call after each successful solver step. Needed to enforce
% the ODE solver time limit.
outputFun = @(t,y,flag)solverTimeLimitFun(t, y, flag, result.odeSettings.solverTimeLimit);
options = odeset('OutputFcn', outputFun);

% Pre-allocate arrays for speed based on the output variable list 
result.modelOutput = zeros(length(result.odeSettings.initialConditions), ...
                      result.odeSettings.NR, length(result.odeSettings.tspan));

% These will be used to contain warnings from ODE solver failures, including
% our ODE solver time limit facility.
savedWarnMsg = '';
savedWarnId = '';

% These might not be defined if an ODE solver error occurs before the first
% successful integration step. Our error handling requires that they be
% defined.
t = [];
y = [];

% Perform the NR model runs.
for run = 1:result.odeSettings.NR
    fprintf('%d ', run);
    
    if (mod(run, 25) == 0)
        fprintf('\n');
    end

    runParameters = result.paramMatrix(run,:);
    if isfield(result.odeSettings, 'timeCourseValues')
        runParameters = appendTimeCourse(runParameters, result.odeSettings.tspan, ...
                                         result.odeSettings.timeCourseValues);
    end
    y0 = result.icMatrix(run,:);
    
    
    try
        if ~isfield(result.odeSettings, 'pulseTimePoints')
            % Not pulsing ODE conditions, just run the model.
            lastwarn('') % Clear last warning message
            [t,y] = ode15s(@(t,y)result.odeSettings.odeModelHandle(t, y, runParameters), result.odeSettings.tspan, y0, options);
            [savedWarnMsg, savedWarnId] = checkWarning();
        else
            % Pulsing ODE conditions. Update the initial conditions at the
            % pulse time step.
            
            % Solve for the time steps up to just before the first pulse.
            preTspan = result.odeSettings.tspan(1:result.odeSettings.pulseTimePointIdx(1) - 1);
            lastwarn('') % Clear last warning message
            [t,y] = ode15s(@(t,y)result.odeSettings.odeModelHandle(t, y, runParameters), preTspan, y0, options);
            [savedWarnMsg, savedWarnId] = checkWarning();
            
            % Only do the pulses if the initial ODE solver call succeeded.
            if (length(t) == length(preTspan))

                % Initial conditions for first pulse solver call, the ODE state
                % at the end of this pre-pulse solver call.
                y0 = y(end,:);

                for i = 1:length(result.odeSettings.pulseTimePointIdx)
                    
                    % ptpidx and ptpidxNext are indices into tspan.
                    ptpidx = result.odeSettings.pulseTimePointIdx(i);
                    if i < length(result.odeSettings.pulseTimePointIdx)
                        % About to do the next pulse. Solve to the time point
                        % just before the time point of the next pulse.
                        ptpidxNext = result.odeSettings.pulseTimePointIdx(i+1) - 1;
                    else
                        % About to do the last pulse. Solve to the end of the
                        % time points.
                        ptpidxNext = length(result.odeSettings.tspan);
                    end
                    
                    % Pulse.
                    % pulseMatrix is 3D, so is pulseMatrix(run,i,:), y0 2D.
                    pulseVector = reshape(pulseMatrix(run,i,:), size(y0));
                    y0 = y0 + pulseVector;
                   
                    % Solve the ODE model from the end of the last solution to
                    % the time of the next pulse. Note that the 1st time point
                    % is the same as the last time point for the last solver
                    % call, since the model is not solved for that 1st time
                    % point - the initial conditions passed to the solver are
                    % also the 1st model output, for that 1st time point.
                    currTspan = result.odeSettings.tspan(ptpidx - 1:ptpidxNext);
                    
                    lastwarn('') % Clear last warning message
                    [tCurr,yCurr] = ode15s(@(tCurr,yCurr)result.odeSettings.odeModelHandle(tCurr, yCurr, runParameters), currTspan, y0, options);
                    [savedWarnMsg, savedWarnId] = checkWarning();

    
                    % Append the current result vectors into final result
                    % vectors. Skip the first time point and first model
                    % output. These are the same as for the end of the prior
                    % solver call.
                    t = [t;tCurr(2:end)];
                    y = [y;yCurr(2:end,:)];

                    % Check if the ode solver stopped early.
                    if (length(tCurr) ~= length(currTspan))
                        break;
                    end                    
    
                    % The initial conditions for the next solver call are the
                    % ODE state at the last time step (i.e. end) of the prior
                    % solver call.                
                    y0 = yCurr(end,:);
                end % end for i = 1:length(result.odeSettings.pulseTimePointIdx)
            end
        end
    catch err
        % Save the error info so it can be included in error_runs later.
        savedWarnMsg = err.message;
        savedWarnId = err.identifier;
    end
    
    if (length(t) == length(result.odeSettings.tspan))
        % Store ode outputs. Transpose y because y is TP x O and odeOut is O x
        % R x TP.
        result.modelOutput(:, run, :) = y';

        % Put current value in base workspace so the cleanup function can
        % save it.
        assignin('base', 'result', result);
    else
        % The ode solver stopped early.
        result.error_runs{end+1} = {run, savedWarnMsg, savedWarnId};
        assignin('base', 'result', result);
        if (result.odeSettings.saveOnError == 1) 
            % Pad with last value and record.
            result.modelOutput = saveError(result.modelOutput, run, y, result.odeSettings.tspan);
            
            % Put current value in base workspace so the cleanup function can
            % save it.
            assignin('base', 'result', result);
        end    
    end
end
fprintf('\n');

% Put current value in base workspace so the cleanup function can save it.
assignin('base', 'result', result);

% Do the PRCC, if requested (alpha exists in the ode settings).
%
% Only if doing at least 2 runs and have at least 2 varying parameters.
% If fewer than 2 varying parameters then a fatal Matlab error would be
% generated in the PRCC analysis. Also don't do PRCC if any parameter sets
% had ODE solver errors.

if (~isfield(result.odeSettings, 'alpha'))
    return;
end

if result.odeSettings.NR < 2
    msg = ['Skipping PRCC - not enough runs.', ...
           'There are only %d runs but at least 2 are required.\n'
            ];
    fprintf(msg, result.odeSettings.NR);
    return;
elseif length(result.paramNames) < 2
    msg = 'Skipping PRCC - not enough varying items to do a PRCC.\n';
    fprintf(msg);    
    return;
elseif ~ isempty(result.error_runs)
    msg = 'Skipping PRCC - ODE solver errors occurred for %d parameter sets.\n';
    fprintf(msg, length(result.error_runs));    
    return;
end

% The PRCC analysis code requires the structure it is passed to have the
% following fields.  It won't know about result.odeSettings and its fields.
result.analysisTimePoints = result.odeSettings.analysisTimePoints;
result.alpha = result.odeSettings.alpha;
result.modelOutputNames = result.odeSettings.icNames;

result = performPrccWithCorrections(result);
assignin('base', 'result', result);
        
fprintf('\n=====================================\n');
fprintf('   End ODE LHS Run');
fprintf('\n=====================================\n');

end

function status = solverTimeLimitFun(t,y,flag, solverTimeLimit)   %#ok<INUSL>

% This function gets called after each successful ODE solver integration step.
% It checks the elapsed time from the call to the solver to see if it is over
% the limit.
% Don't throw an error here because that will also trigger the clean up
% function, which will stop processing subsequent parameter sets. We just
% want to halt the solver on those parameter sets that take too long, and
% continue with all the others.

% Instead, issue a warning, since that will be handled by the same error
% handling code that handles warnings from the ODE solver.

persistent INIT_TIME;
status = 0;
switch(flag)
    case 'init'
        INIT_TIME = tic;
    case 'done'
        clear INIT_TIME;
    otherwise
        elapsed_time = toc(INIT_TIME);
        if elapsed_time > solverTimeLimit
            clear INIT_TIME;
            status = 1;
            msg = sprintf('Exceeded time limit of %f, elapsed time is %f,\n',...
                    solverTimeLimit, elapsed_time);
            warning('LHS_ODE_RUN:SolverTimeLimitElapsed', msg);
        end
end
end

function [savedWarnMsg, savedWarnId] = checkWarning()

% Save any non-empty warning so the caller can associate it with any ODE solver
% error.

savedWarnMsg = '';
savedWarnId = '';
[warnMsg, warnId] = lastwarn;
if ~isempty(warnMsg)
    savedWarnMsg = warnMsg;
    savedWarnId = warnId;
end            
end

function cleanMeUp()

% The items from the main function (lhs_ode_run_new) that are to be saved.
% The main function must put these in the base workspace first, otherwise
% they will not be accessible here. Specifically, they are not in the
% caller workspace, since the main function is not the direct caller of
% this clean up function.

result = evalin('base', 'result');

[file, path] = uiputfile('*.mat', 'Save Workspace As', 'Model_LHS.mat');

% Make sure a file name was returned.
if (ischar(file))
    tmpFileStr = strcat(path, file);
    save(tmpFileStr, 'result');
end

end

function runParameters = appendTimeCourse(runParameters, tspan, ...
                                          timeCourseValues)

% Append tspan and corresponding time course values to the run parameters.

% Inputs
%
% runParameters: The model parameters, a vector of size P.
%
% tspan: A vector of size T with the time points the ODE model is to be solved
% for.
%
% timeCourseValues: An S x T matrix of time course values, an S x T matrix
%                   where S is the number of statesdefined by a time course and
%                   T is the same number of elements in tspan, i.e. a value is
%                   defined for each state being time coursed for each time
%                   point.
%
% Outputs
%

% runParameters: The original runParameters with the time steps of tspan and
%                the mode state time courses appended.
%                For example, if on input:
%                runParameters = [7.7, 8.8];
%                tspan = [1.0, 2.0, 3.0];
%                timeCourseValues = [  44.4,  55.5,  66.6; ...
%                                     110.0, 220.0, 330.0 ];
%                Then on output:
%                runParameters = [7.7, 8.8, 3, 1.0, 44.4, 110.0, 2.0, 55.5, 220.0, 3.0, 66.6, 330.0];
%                The value of 3 just after 8.8 is the number of values appended
%                per timepoint, a timepoint value and two state values, 1 + the
%                number of state time course values per timepoint, which is 
%                1 + number of rows in timeCourseValues.

[nrows, ncols] = size(timeCourseValues);
numItemsAdd = length(tspan);

if ncols ~= numItemsAdd
    fmt = 'The number of time course values per time point, %d,\n';
    fmt = [fmt, 'is not the same as the number of time points, %d\n'];
    msg = sprintf(fmt, ncols, numItemsAdd);
    ident = 'VALUEVALIDATION:MismatchedTimepointSizes';
    throw(MException(ident, msg));
end

elementsPerItem = nrows + 1;
numElementsAdd = elementsPerItem * numItemsAdd;
runParameters = [runParameters, elementsPerItem];
addBase = length(runParameters);
runParameters = [runParameters, zeros(1, numElementsAdd)];
for i = 1:numItemsAdd
    j = (i-1)*elementsPerItem+1;
    runParameters(addBase + j) = tspan(i);
    runParameters(addBase + j + 1:addBase + j + nrows) = timeCourseValues(:,i);
end

end

function odeOut = saveError(odeOut, run, y, tspan)
%============================================================================

% When an error occurs, fill odeOut with what model output was produced by the
% solver, and pad the missing output with the values from the last time point
% for which a value was produced by the solver.
%
% Inputs:
%
%   odeOut: A cell array that holds model output.
%
%   run: Current run number.
%
%   y: The output vector from the solver for the current run.
%
%   tspan: Time vector of all time points for which output is normally
%          produced.
%
% Output:
%
%  odeOut: Updated with the model output y, and padded with missing model
%          output.
%
%============================================================================


% timePointCount is the number of time points for which the solver produced
% output.
%
% outputCount is the number of model output variables (number of model
% equations).
%
% tspanCount is the number of timepoints the solver would ahve produced if it
% hadn't quite early.
[timePointCount, outputCount] = size(y);
tspanCount = length(tspan);

% padRowCount is the number of row we need to add to y to bring it to the full
% size.
padRowCount = tspanCount - timePointCount;
padOnes = ones(padRowCount, outputCount);

% Replicate the last row of y padRowCount times, append to y and then store in
% odeOut for this run.
pad = padOnes .* y(end, :);
yPadded = [y; pad];


% Transpose yPadded because yPadded is TP x O and odeOut is O x R x TP.
odeOut(:, run, :) = yPadded';

end % function saveError

